<?php
/**
* @license		GNU/GPL, see LICENSE.php
* @author		Benjamin Jeanjean <benjamin.jeanjean@gmail.com>
*/
defined( '_JEXEC' ) or die( 'Restricted access' );

require_once dirname(__FILE__).DS.'helper.php';

/*******************************
 * Getting module params
 ******************************/
$link_image				= $params->get( 'link_image' );
$recurse	    		= (bool) $params->get( 'recurse_directory' );
$type_open    			= $params->get( 'type_open' );
$foldersExcluded 		= $params->get( 'folders_excluded' );
$moduleclass_sfx 		= $params->get( 'moduleclass_sfx' );
$folder       			= $params->get( 'folder', 'images');
$type_link				= $params->get( 'display_mode' );
$type_url				= $params->get( 'type_url' );
$display_mode			= $params->get( 'display_mode' );
$type_filter	   		= $params->get( 'type_filter', 'jpg png gif' );
$filter_simple    		= $params->get( 'filter_simple', 'jpg png gif' );
$filter_regexp    		= $params->get( 'filter_regexp', '\.(png|jpg|gif)' );
$debug					= (int) $params->get( 'debug', 0 );

/*******************************
 * Sanitize folder param
 ******************************/
$folderSanitized 	= JPath::clean( $folder );
$folderSanitized	= ltrim($folderSanitized, '\/');
$folderSanitized	= rtrim($folderSanitized, '\/');
$fullPathToImages 	= JPATH_SITE . '/' . $folderSanitized;

/*******************************
 * Control if folder param exists and is readable
 ******************************/
if (!is_dir($fullPathToImages)) {
    JError::raiseWarning(21, 
		'Error "Module Random Image +"<br />' . 
		'The "Image folder" you specified in the module manager parameters doesn\'t exists ! -- You have to check your module\'s configuration to make it working.<br />' .
		'Value specified: ' . JURI::root() . $params->get('folder'));
    return;
} elseif (!is_readable($fullPathToImages)) {
	JError::raiseWarning(21, 
		'Error "Module Random Image +"<br />' . 
		'The "Image folder" you specified in the module manager parameters is not readable on hard drive ! -- You have to check your module\'s configuration to make it working or CHMOD your folder.<br />' . 
		'Value specified: ' . JURI::root() . $params->get('folder'));
    return;
}

/*******************************
 * Sanitize image types param
 ******************************/
if ($type_filter == 'simple') {
	$typesSanitized = array();
	$types			= str_replace(' ', ',', $filter_simple); 		// Compatibility with old versions
	$typesArray 	= explode(',', $types);
	foreach($typesArray as $type) {
		$typeSanitized = JString::trim($type);
		if (!empty($typeSanitized)) {
			$typesSanitized[] = $typeSanitized;
		}
	}
	$extFilter	= "\.(" . implode('|', $typesSanitized) . ")";
} else {
	$extFilter = $filter_regexp;
}

/*******************************
 * Sanitize folders excluded param
 ******************************/
$foldersExcluded = explode(',', $foldersExcluded);
$foldersExcluded[] = '.svn';
$foldersExcluded[] = 'CVS';
foreach ($foldersExcluded as $folderExcluded) {
	$folderExcludedSanitized = JString::trim($folderExcluded);
	if (!empty($folderExcludedSanitized)) {
		$foldersExcludedSanitized[] = $folderExcludedSanitized;
	}
}

/*******************************
 * Getting the images from params
 ******************************/
$images 		= JFolderWCP::files(
					$fullPathToImages, 
					$extFilter, 
					$recurse, 
					true, 
					$foldersExcludedSanitized);

// If the folder has no images, display an error
if (count($images) == 0) {
	JError::raiseWarning(21, 
		'Error "Module Random Image +"<br />' . 
		'There are no images in this folder -- You have to choose another folder which contains images or maybe you have to be less restrictive for the images types (for simple or regexp filter mode)!<br />' .
		'Value specified (folder) : ' . $params->get('folder') . '<br />' .
		'Value specified (images types) : ' . $params->get('types'));
	return;
}

/*******************************
 * Getting random image
 ******************************/
$nbimages	 	= count($images);
$random 		= mt_rand(0, $nbimages - 1);
$randImage 		= $images[$random];
$randImage		= JPath::clean($randImage);
					
/*******************************
 * Calculate size for the image
 ******************************/
$imgSize 		= getimagesize ($randImage);
$widthParam		= $params->get('size_width');
$heightParam	= $params->get('size_height');

switch ($params->get('type_scale')) {
	case 'width' : 
		$sizeCoeff 	= $imgSize[0] / $imgSize[1];
		$widthImg 	= $widthParam;
		$heightImg 	= (int) ($widthParam / $sizeCoeff);
		break;
	case 'height' : 
		if ($imgSize[1] > $imgSize[0]) {
			$sizeCoeff = $imgSize[1] / $imgSize[0];
		} else {
			$sizeCoeff = $imgSize[1] / $imgSize[0];
		}
		$widthImg 	= (int) ($heightParam / $sizeCoeff);
		$heightImg 	= $heightParam;
		break;
	case 'resize' : 
		$widthImg 	= $widthParam;
		$heightImg 	= $heightParam;
		break;
	case 'none' :
	default:
		$widthImg  = $size[0];
		$heightImg = $size[1];
}

/*******************************
 * Construct final image array
 ******************************/
$img 			= new modRandomImagePlusImage($randImage);
$img->width 	= $widthImg;
$img->height 	= $heightImg;
$img->filename 	= basename($randImage);

if ($img->uri === false) {
    JError::raiseWarning(21, 
		'Error "Module Random Image +"<br />' . 
		'Cannot resolve URI (' . $uri . ')');
    return;
}

/*******************************
 * Manage link type 
 ******************************/
if ($type_link == 'image_text') {
	$img->link_text = $params->get('link_name');
}

/*******************************
 * Manage link url
 ******************************/
if ($type_url == 'link_url') {
	$img->anchor_uri = $params->get('link_href');
} else {
	$img->anchor_uri = $img->uri;
}

/*******************************
 * Insert Javascript for SqueezeBox Mode
 ******************************/
if ($type_open == 'squeezebox') {
    $document =& JFactory::getDocument();
    $document->addScript(JURI::base().'media/system/js/modal.js');
    $document->addStyleSheet(JURI::base().'media/system/css/modal.css');
    $document->addScriptDeclaration("
		window.addEvent('domready', function() {
			SqueezeBox.initialize({});
			$$('a.mod_ran_img_plus_modal').each(function(el) {
				el.addEvent('click', function(e) {
					new Event(e).stop();
					SqueezeBox.fromElement(el);
				});
			});
		});");
}

/*******************************
 * Prepare data for debug mode
 ******************************/
if ($debug == 1) {
	$debugData = array();
	foreach ($images as $image) {
		$debugDatas[] = str_replace(JPATH_SITE, '', $image);
	}
}

/*******************************
 * Construct anchor attributes
 ******************************/
if ($type_open == 'squeezebox') {
	$anchorAttr = 'class="mod_ran_img_plus_modal" rel="SqueezeBox"';
} elseif ($type_open == 'new_window') {
	$anchorAttr = 'target="_blank"';
} else {
	$anchorAttr = 'target="_parent"';
}
 
/*******************************
 * Go render !
 ******************************/
require(JModuleHelper::getLayoutPath('mod_random_image_plus'));